import importPlugin from 'eslint-plugin-import-x';
import noLookaheadLookbehindRegexp from 'eslint-plugin-no-lookahead-lookbehind-regexp';
import globals from 'globals';

export default [
  {
    plugins: {
      import: importPlugin,
      'no-lookahead-lookbehind-regexp': noLookaheadLookbehindRegexp,
    },

    linterOptions: {
      reportUnusedDisableDirectives: true,
    },

    languageOptions: {
      globals: {
        ...globals['shared-node-browser'],
      },
    },

    rules: {
      // https://eslint.org/docs/latest/rules/
      // Last rules review: v8.56.0

      // https://github.com/import-js/eslint-plugin-import#rules
      // Last rules review: v2.26.0

      //#region Possible Problems (https://eslint.org/docs/latest/rules/#possible-problems)
      'array-callback-return': 'error',
      'constructor-super': 'error',
      'for-direction': 'error',
      'getter-return': 'error',
      'no-async-promise-executor': 'error',
      'no-await-in-loop': 'error',
      'no-class-assign': 'error',
      'no-compare-neg-zero': 'error',
      'no-cond-assign': 'error',
      'no-const-assign': 'error',
      'no-constant-binary-expression': 'error',
      'no-constant-condition': ['error', { checkLoops: false }],
      'no-constructor-return': 'error',
      'no-control-regex': 'error',
      'no-debugger': 'error',
      'no-dupe-args': 'error',
      'no-dupe-class-members': 'error',
      'no-dupe-else-if': 'error',
      'no-dupe-keys': 'error',
      'no-duplicate-case': 'error',
      'no-duplicate-imports': 'off',
      'no-empty-character-class': 'error',
      'no-empty-pattern': 'error',
      'no-ex-assign': 'error',
      'no-fallthrough': 'error',
      'no-func-assign': 'error',
      'no-import-assign': 'error',
      'no-inner-declarations': 'error',
      'no-invalid-regexp': 'error',
      'no-irregular-whitespace': 'error',
      'no-loss-of-precision': 'error',
      'no-misleading-character-class': 'error',
      'no-new-native-nonconstructor': 'error',
      'no-new-symbol': 'error',
      'no-obj-calls': 'error',
      'no-promise-executor-return': 'error',
      'no-prototype-builtins': 'error',
      'no-self-assign': 'error',
      'no-self-compare': 'error',
      'no-setter-return': 'error',
      'no-sparse-arrays': 'error',
      'no-template-curly-in-string': 'error',
      'no-this-before-super': 'error',
      'no-undef': 'error',
      'no-unexpected-multiline': 'error',
      'no-unmodified-loop-condition': 'error',
      'no-unreachable': 'error',
      'no-unreachable-loop': 'error',
      'no-unsafe-finally': 'error',
      'no-unsafe-negation': 'error',
      'no-unsafe-optional-chaining': 'error',
      'no-unused-private-class-members': 'error',
      'no-unused-vars': 'error',
      'no-use-before-define': 'off',
      'no-useless-backreference': 'error',
      'require-atomic-updates': ['error', { allowProperties: true }],
      'use-isnan': 'error',
      'valid-typeof': 'error',
      //#endregion

      //#region Suggestions (https://eslint.org/docs/latest/rules/#suggestions)
      'accessor-pairs': 'off',
      'arrow-body-style': 'off',
      'block-scoped-var': 'error',
      camelcase: ['error', { properties: 'always', ignoreDestructuring: true }],
      'capitalized-comments': 'off',
      'class-methods-use-this': 'off',
      complexity: 'off',
      'consistent-return': 'off',
      'consistent-this': 'off',
      curly: ['error', 'multi-line', 'consistent'],
      'default-case': 'error',
      'default-case-last': 'error',
      'default-param-last': 'error',
      'dot-notation': 'error',
      eqeqeq: ['error', 'allow-null'],
      'func-name-matching': 'off',
      'func-names': 'error',
      'func-style': 'off',
      'grouped-accessor-pairs': 'error',
      'guard-for-in': 'off',
      'id-denylist': 'off',
      'id-length': 'off',
      'id-match': 'off',
      'init-declarations': 'off',
      'logical-assignment-operators': 'off',
      'max-classes-per-file': 'off',
      'max-depth': 'off',
      'max-lines': 'off',
      'max-lines-per-function': 'off',
      'max-nested-callbacks': 'off',
      'max-params': 'off',
      'max-statements': 'off',
      'multiline-comment-style': 'off',
      'new-cap': 'error',
      'no-alert': 'error',
      'no-array-constructor': 'error',
      'no-bitwise': 'off',
      'no-caller': 'error',
      'no-case-declarations': 'error',
      'no-console': 'error',
      'no-continue': 'off',
      'no-delete-var': 'error',
      'no-div-regex': 'error',
      'no-else-return': 'off',
      'no-empty': 'error',
      'no-empty-function': 'warn',
      'no-empty-static-block': 'error',
      'no-eq-null': 'off',
      'no-eval': 'error',
      'no-extend-native': 'error',
      'no-extra-bind': 'error',
      'no-extra-boolean-cast': 'error',
      'no-extra-label': 'error',
      'no-global-assign': 'error',
      'no-implicit-coercion': 'off',
      'no-implicit-globals': 'error',
      'no-implied-eval': 'error',
      'no-inline-comments': 'off',
      'no-invalid-this': 'error',
      'no-iterator': 'error',
      'no-label-var': 'error',
      'no-labels': 'off',
      'no-lone-blocks': 'error',
      'no-lonely-if': 'error',
      'no-loop-func': 'error',
      'no-magic-numbers': 'off',
      'no-multi-assign': 'error',
      'no-multi-str': 'error',
      'no-negated-condition': 'off',
      'no-nested-ternary': 'off',
      'no-new': 'error',
      'no-new-func': 'error',
      'no-new-wrappers': 'error',
      'no-nonoctal-decimal-escape': 'error',
      'no-object-constructor': 'error',
      'no-octal': 'error',
      'no-octal-escape': 'error',
      'no-param-reassign': 'off',
      'no-plusplus': 'off',
      'no-proto': 'error',
      'no-redeclare': 'error',
      'no-regex-spaces': 'error',
      'no-restricted-exports': 'off',
      'no-restricted-globals': 'off',
      'no-restricted-imports': 'off',
      'no-restricted-properties': 'off',
      'no-restricted-syntax': 'off',
      'no-return-assign': 'error',
      'no-script-url': 'error',
      'no-sequences': 'error',
      'no-shadow': 'off',
      'no-shadow-restricted-names': 'error',
      'no-ternary': 'off',
      'no-throw-literal': 'error',
      'no-undef-init': 'error',
      'no-undefined': 'off',
      'no-underscore-dangle': 'off',
      'no-unneeded-ternary': 'error',
      'no-unused-expressions': 'error',
      'no-unused-labels': 'error',
      'no-useless-call': 'error',
      'no-useless-catch': 'error',
      'no-useless-computed-key': 'error',
      'no-useless-concat': 'error',
      'no-useless-constructor': 'error',
      'no-useless-escape': 'error',
      'no-useless-rename': 'error',
      'no-useless-return': 'error',
      'no-var': 'error',
      // Conflicts with typescript-eslint's no-floating-promises rule.
      'no-void': 'off',
      'no-warning-comments': 'warn',
      'no-with': 'error',
      'object-shorthand': 'error',
      'one-var': ['error', { initialized: 'never' }],
      'operator-assignment': 'off',
      'prefer-arrow-callback': 'off',
      'prefer-const': 'off',
      'prefer-destructuring': 'off',
      'prefer-exponentiation-operator': 'error',
      'prefer-named-capture-group': 'warn',
      'prefer-numeric-literals': 'error',
      'prefer-object-has-own': 'error',
      'prefer-object-spread': 'error',
      'prefer-promise-reject-errors': 'error',
      'prefer-regex-literals': 'error',
      'prefer-rest-params': 'error',
      'prefer-spread': 'error',
      'prefer-template': 'error',
      radix: 'error',
      'require-await': 'off',
      'require-unicode-regexp': 'off',
      'require-yield': 'error',
      'sort-imports': 'off',
      'sort-keys': 'off',
      'sort-vars': 'off',
      strict: ['error', 'global'],
      'symbol-description': 'error',
      'vars-on-top': 'off',
      yoda: 'error',
      //#endregion

      //#region Layout & Formatting (https://eslint.org/docs/latest/rules/#layout--formatting)
      'line-comment-position': 'off',
      'unicode-bom': 'error',
      //#endregion

      //#region Import plugin (https://github.com/import-js/eslint-plugin-import#rules)
      // Static analysis
      'import/no-absolute-path': 'error',
      'import/no-webpack-loader-syntax': 'error',
      'import/no-self-import': 'error',
      'import/no-useless-path-segments': 'error',

      // Module systems
      'import/no-amd': 'error',

      // Style guide
      'import/first': 'error',
      'import/no-duplicates': 'error',
      'import/order': [
        'warn',
        {
          groups: [
            'builtin',
            'external',
            'internal',
            'parent',
            'sibling',
            'index',
          ],
          'newlines-between': 'always',
          alphabetize: {
            order: 'asc',
          },
        },
      ],
      'import/newline-after-import': 'error',
      'import/no-unassigned-import': [
        'warn',
        {
          allow: [
            'make-promises-safe',
            'node-report',
            'reflect-metadata',
            '**/*.css',
            'react-app-polyfill/*',
          ],
        },
      ],
      'import/no-named-default': 'error',
      //#endregion

      //#region No lookbehind plugin (https://github.com/JonasBa/eslint-plugin-no-lookahead-lookbehind-regexp)
      'no-lookahead-lookbehind-regexp/no-lookahead-lookbehind-regexp': [
        'error',
        'no-lookbehind',
        'no-negative-lookbehind',
        { browserslist: false },
      ],
      //#endregion
    },
  },
  {
    // .cjs files are always for Node.js.
    files: ['**/*.cjs'],
    languageOptions: {
      globals: {
        ...globals.node,
      },
      sourceType: 'commonjs',
    },
  },
  {
    // Configuration files are usually for Node.js modules.
    files: ['**/*.config.{js,cjs,mjs,ts,cts,mts}'],
    languageOptions: {
      globals: {
        ...globals.node,
      },
    },
  },
];
